var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
import { RepositoryNotFoundError } from "./error/RepositoryNotFoundError";
import { EntityListenerMetadata } from "../metadata/EntityListenerMetadata";
// import {EntityManager} from "../entity-manager/EntityManager";
// import {importClassesFromDirectories, importJsonsFromDirectories} from "../util/DirectoryExportedClassesLoader";
import { getFromContainer } from "../container";
import { getMetadataArgsStorage } from "../../index";
import { EntityMetadataBuilder } from "../metadata-builder/EntityMetadataBuilder";
import { DefaultNamingStrategy } from "../naming-strategy/DefaultNamingStrategy";
import { EntityMetadataCollection } from "../metadata-args/collection/EntityMetadataCollection";
import { NoConnectionForRepositoryError } from "./error/NoConnectionForRepositoryError";
import { CannotImportAlreadyConnectedError } from "./error/CannotImportAlreadyConnectedError";
import { CannotCloseNotConnectedError } from "./error/CannotCloseNotConnectedError";
import { CannotConnectAlreadyConnectedError } from "./error/CannotConnectAlreadyConnectedError";
import { NamingStrategyNotFoundError } from "./error/NamingStrategyNotFoundError";
import { RepositoryNotTreeError } from "./error/RepositoryNotTreeError";
// import {EntitySchema} from "../entity-schema/EntitySchema";
import { CannotSyncNotConnectedError } from "./error/CannotSyncNotConnectedError";
import { CannotUseNamingStrategyNotConnectedError } from "./error/CannotUseNamingStrategyNotConnectedError";
import { Broadcaster } from "../subscriber/Broadcaster";
import { LazyRelationsWrapper } from "../lazy-loading/LazyRelationsWrapper";
import { SchemaBuilder } from "../schema-builder/SchemaBuilder";
import { EntityManager } from "../entity-manager/EntityManager";
import { RepositoryAggregator } from "../repository/RepositoryAggregator";
import { QueryRunnerProvider } from "../query-runner/QueryRunnerProvider";
// import {QueryRunnerProvider} from "../query-runner/QueryRunnerProvider";
/**
 * Connection is a single database connection to a specific database of a database management system.
 * You can have multiple connections to multiple databases in your application.
 */
var Connection = (function () {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    function Connection(name, driver, logger) {
        /**
         * All entity metadatas that are registered for this connection.
         */
        this.entityMetadatas = new EntityMetadataCollection();
        /**
         * Stores all registered repositories.
         */
        this.repositoryAggregators = [];
        /**
         * Entity listeners that are registered for this connection.
         */
        this.entityListeners = [];
        /**
         * Entity subscribers that are registered for this connection.
         */
        this.entitySubscribers = [];
        /**
         * Registered entity classes to be used for this connection.
         */
        this.entityClasses = [];
        /**
         * Registered entity schemas to be used for this connection.
         */
        this.entitySchemas = [];
        /**
         * Registered subscriber classes to be used for this connection.
         */
        this.subscriberClasses = [];
        /**
         * Registered naming strategy classes to be used for this connection.
         */
        this.namingStrategyClasses = [];
        /**
         * Indicates if connection has been done or not.
         */
        this._isConnected = false;
        this.name = name;
        this.driver = driver;
        this.logger = logger;
        this._entityManager = this.createEntityManager();
        this.broadcaster = this.createBroadcaster();
    }
    Object.defineProperty(Connection.prototype, "isConnected", {
        // -------------------------------------------------------------------------
        // Accessors
        // -------------------------------------------------------------------------
        /**
         * Indicates if connection to the database already established for this connection.
         */
        get: function () {
            return this._isConnected;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Gets entity manager that allows to perform repository operations with any entity in this connection.
     */
    /*get entityManager() {
        if (!this.isConnected)
            throw new CannotGetEntityManagerNotConnectedError(this.name);
        
        return this._entityManager;
    }*/
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    /**
     * Performs connection to the database.
     */
    Connection.prototype.connect = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.isConnected)
                            throw new CannotConnectAlreadyConnectedError(this.name);
                        // connect to the database via its driver
                        return [4 /*yield*/, this.driver.connect()];
                    case 1:
                        // connect to the database via its driver
                        _a.sent();
                        // build all metadatas registered in the current connection
                        this.buildMetadatas();
                        // set connected status for the current connection
                        this._isConnected = true;
                        return [2 /*return*/, this];
                }
            });
        });
    };
    /**
     * Closes connection with the database.
     * Once connection is closed, you cannot use repositories and perform any operations except
     * opening connection again.
     */
    Connection.prototype.close = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.isConnected)
                            throw new CannotCloseNotConnectedError(this.name);
                        return [4 /*yield*/, this.driver.disconnect()];
                    case 1:
                        _a.sent();
                        this._isConnected = false;
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Drops the database and all its data.
     */
    Connection.prototype.dropDatabase = function () {
        return __awaiter(this, void 0, void 0, function () {
            var queryRunner, error_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.driver.createQueryRunner()];
                    case 1:
                        queryRunner = _a.sent();
                        return [4 /*yield*/, queryRunner.beginTransaction()];
                    case 2:
                        _a.sent();
                        _a.label = 3;
                    case 3:
                        _a.trys.push([3, 7, , 10]);
                        return [4 /*yield*/, queryRunner.clearDatabase()];
                    case 4:
                        _a.sent();
                        return [4 /*yield*/, queryRunner.commitTransaction()];
                    case 5:
                        _a.sent();
                        return [4 /*yield*/, queryRunner.release()];
                    case 6:
                        _a.sent();
                        return [3 /*break*/, 10];
                    case 7:
                        error_1 = _a.sent();
                        return [4 /*yield*/, queryRunner.rollbackTransaction()];
                    case 8:
                        _a.sent();
                        return [4 /*yield*/, queryRunner.release()];
                    case 9:
                        _a.sent();
                        throw error_1;
                    case 10: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Creates database schema for all entities registered in this connection.
     *
     * @param dropBeforeSync If set to true then it drops the database with all its tables and data
     */
    Connection.prototype.syncSchema = function (dropBeforeSync) {
        if (dropBeforeSync === void 0) { dropBeforeSync = false; }
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.isConnected)
                            return [2 /*return*/, Promise.reject(new CannotSyncNotConnectedError(this.name))];
                        if (!dropBeforeSync) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.dropDatabase()];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2: return [4 /*yield*/, this.createSchemaBuilder().build()];
                    case 3:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Imports entities from the given paths (directories) and registers them in the current connection.
     */
    /*importEntitiesFromDirectories(paths: string[]): this {
        this.importEntities(importClassesFromDirectories(paths));
        return this;
    }

    /!**
     * Imports entity schemas from the given paths (directories) and registers them in the current connection.
     *!/
    importEntitySchemaFromDirectories(paths: string[]): this {
        this.importEntitySchemas(importJsonsFromDirectories(paths));
        return this;
    }

    /!**
     * Imports subscribers from the given paths (directories) and registers them in the current connection.
     *!/
    importSubscribersFromDirectories(paths: string[]): this {
        this.importSubscribers(importClassesFromDirectories(paths));
        return this;
    }

    /!**
     * Imports naming strategies from the given paths (directories) and registers them in the current connection.
     *!/
    importNamingStrategiesFromDirectories(paths: string[]): this {
        this.importEntities(importClassesFromDirectories(paths));
        return this;
    }*/
    /**
     * Imports entities and registers them in the current connection.
     */
    Connection.prototype.importEntities = function (entities) {
        var _this = this;
        if (this.isConnected)
            throw new CannotImportAlreadyConnectedError("entities", this.name);
        entities.forEach(function (cls) { return _this.entityClasses.push(cls); });
        return this;
    };
    /**
     * Imports schemas and registers them in the current connection.
     */
    Connection.prototype.importEntitySchemas = function (schemas) {
        var _this = this;
        if (this.isConnected)
            throw new CannotImportAlreadyConnectedError("schemas", this.name);
        schemas.forEach(function (schema) { return _this.entitySchemas.push(schema); });
        return this;
    };
    /**
     * Imports subscribers and registers them in the current connection.
     */
    Connection.prototype.importSubscribers = function (subscriberClasses) {
        var _this = this;
        if (this.isConnected)
            throw new CannotImportAlreadyConnectedError("entity subscribers", this.name);
        subscriberClasses.forEach(function (cls) { return _this.subscriberClasses.push(cls); });
        return this;
    };
    /**
     * Imports naming strategies and registers them in the current connection.
     */
    Connection.prototype.importNamingStrategies = function (strategies) {
        var _this = this;
        if (this.isConnected)
            throw new CannotImportAlreadyConnectedError("naming strategies", this.name);
        strategies.forEach(function (cls) { return _this.namingStrategyClasses.push(cls); });
        return this;
    };
    /**
     * Sets given naming strategy to be used.
     * Naming strategy must be set to be used before connection is established.
     */
    Connection.prototype.useNamingStrategy = function (strategyClassOrName) {
        if (this.isConnected)
            throw new CannotUseNamingStrategyNotConnectedError(this.name);
        this.usedNamingStrategy = strategyClassOrName;
        return this;
    };
    /**
     Gets entity metadata for the given entity class or schema name.
     */
    Connection.prototype.getMetadata = function (entity) {
        return this.entityMetadatas.findByTarget(entity);
    };
    /**
     * Gets repository for the given entity class or name.
     */
    Connection.prototype.getRepository = function (entityClassOrName) {
        return this.findRepositoryAggregator(entityClassOrName).repository;
    };
    /**
     * Gets tree repository for the given entity class or name.
     * Only tree-type entities can have a TreeRepository,
     * like ones decorated with @ClosureTable decorator.
     */
    Connection.prototype.getTreeRepository = function (entityClassOrName) {
        var repository = this.findRepositoryAggregator(entityClassOrName).treeRepository;
        if (!repository)
            throw new RepositoryNotTreeError(entityClassOrName);
        return repository;
    };
    /**
     * Gets specific repository for the given entity class or name.
     * SpecificRepository is a special repository that contains specific and non standard repository methods.
     */
    Connection.prototype.getSpecificRepository = function (entityClassOrName) {
        return this.findRepositoryAggregator(entityClassOrName).specificRepository;
    };
    /**
     * Creates a new entity manager with a single opened connection to the database.
     * This may be useful if you want to perform all db queries within one connection.
     * After finishing with entity manager, don't forget to release it, to release connection back to pool.
     */
    Connection.prototype.createEntityManagerWithSingleDatabaseConnection = function () {
        var queryRunnerProvider = new QueryRunnerProvider(this.driver, true);
        return new EntityManager(this, queryRunnerProvider);
    };
    // -------------------------------------------------------------------------
    // Protected Methods
    // -------------------------------------------------------------------------
    /**
     * Finds repository aggregator of the given entity class or name.
     */
    Connection.prototype.findRepositoryAggregator = function (entityClassOrName) {
        if (!this.isConnected)
            throw new NoConnectionForRepositoryError(this.name);
        if (!this.entityMetadatas.hasTarget(entityClassOrName))
            throw new RepositoryNotFoundError(this.name, entityClassOrName);
        var metadata = this.entityMetadatas.findByTarget(entityClassOrName);
        var repositoryAggregator = this.repositoryAggregators.find(function (repositoryAggregate) { return repositoryAggregate.metadata === metadata; });
        if (!repositoryAggregator)
            throw new RepositoryNotFoundError(this.name, entityClassOrName);
        return repositoryAggregator;
    };
    /**
     * Builds all registered metadatas.
     */
    Connection.prototype.buildMetadatas = function () {
        var _this = this;
        this.entitySubscribers.length = 0;
        this.entityListeners.length = 0;
        this.repositoryAggregators.length = 0;
        this.entityMetadatas.length = 0;
        var namingStrategy = this.createNamingStrategy();
        var lazyRelationsWrapper = this.createLazyRelationsWrapper();
        // take imported event subscribers
        if (this.subscriberClasses && this.subscriberClasses.length) {
            getMetadataArgsStorage()
                .entitySubscribers
                .filterByTargets(this.subscriberClasses)
                .map(function (metadata) { return getFromContainer(metadata.target); })
                .forEach(function (subscriber) { return _this.entitySubscribers.push(subscriber); });
        }
        // take imported entity listeners
        if (this.entityClasses && this.entityClasses.length) {
            getMetadataArgsStorage()
                .entityListeners
                .filterByTargets(this.entityClasses)
                .forEach(function (metadata) { return _this.entityListeners.push(new EntityListenerMetadata(metadata)); });
        }
        // build entity metadatas from metadata args storage (collected from decorators)
        if (this.entityClasses && this.entityClasses.length) {
            getFromContainer(EntityMetadataBuilder)
                .buildFromMetadataArgsStorage(this.driver, lazyRelationsWrapper, namingStrategy, this.entityClasses)
                .forEach(function (metadata) {
                _this.entityMetadatas.push(metadata);
                _this.repositoryAggregators.push(new RepositoryAggregator(_this, metadata));
            });
        }
        // build entity metadatas from given entity schemas
        if (this.entitySchemas && this.entitySchemas.length) {
            getFromContainer(EntityMetadataBuilder)
                .buildFromSchemas(this.driver, lazyRelationsWrapper, namingStrategy, this.entitySchemas)
                .forEach(function (metadata) {
                _this.entityMetadatas.push(metadata);
                _this.repositoryAggregators.push(new RepositoryAggregator(_this, metadata));
            });
        }
    };
    /**
     * Creates a naming strategy to be used for this connection.
     */
    Connection.prototype.createNamingStrategy = function () {
        var _this = this;
        // if naming strategies are not loaded, or used naming strategy is not set then use default naming strategy
        if (!this.namingStrategyClasses || !this.namingStrategyClasses.length || !this.usedNamingStrategy)
            return getFromContainer(DefaultNamingStrategy);
        // try to find used naming strategy in the list of loaded naming strategies
        var namingMetadata = getMetadataArgsStorage()
            .namingStrategies
            .filterByTargets(this.namingStrategyClasses)
            .find(function (strategy) {
            if (typeof _this.usedNamingStrategy === "string") {
                return strategy.name === _this.usedNamingStrategy;
            }
            else {
                return strategy.target === _this.usedNamingStrategy;
            }
        });
        // throw an error if not found
        if (!namingMetadata)
            throw new NamingStrategyNotFoundError(this.usedNamingStrategy, this.name);
        // initialize a naming strategy instance
        return getFromContainer(namingMetadata.target);
    };
    /**
     * Creates a new default entity manager without single connection setup.
     */
    Connection.prototype.createEntityManager = function () {
        return new EntityManager(this);
    };
    /**
     * Creates a new entity broadcaster using in this connection.
     */
    Connection.prototype.createBroadcaster = function () {
        return new Broadcaster(this.entityMetadatas, this.entitySubscribers, this.entityListeners);
    };
    /**
     * Creates a schema builder used to build a database schema for the entities of the current connection.
     */
    Connection.prototype.createSchemaBuilder = function () {
        return new SchemaBuilder(this.driver, this.logger, this.entityMetadatas, this.createNamingStrategy());
    };
    /**
     * Creates a lazy relations wrapper.
     */
    Connection.prototype.createLazyRelationsWrapper = function () {
        return new LazyRelationsWrapper(this);
    };
    return Connection;
}());
export { Connection };
//# sourceMappingURL=Connection.js.map