var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var win = window;
import { ConnectionIsNotSetError } from "../error/ConnectionIsNotSetError";
import { DriverPackageNotInstalledError } from "../error/DriverPackageNotInstalledError";
import { ColumnTypes } from "../../metadata/types/ColumnTypes";
import { ColumnMetadata } from "../../metadata/ColumnMetadata";
import moment from 'moment';
import { WebSqlQueryRunner } from "./WebSqlQueryRunner";
/**
 * Organizes communication with websql DBMS.
 */
var WebSqlDriver = (function () {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    function WebSqlDriver(connectionOptions, logger, websql) {
        this.options = connectionOptions;
        this.logger = logger;
        this.websql = websql;
        // validate options to make sure everything is set
        // if (!this.options.storage)
        //     throw new DriverOptionNotSetError("storage");
        // if websql package instance was not set explicitly then try to load it
        if (!websql)
            this.loadDependencies();
    }
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    /**
     * Performs connection to the database.
     */
    WebSqlDriver.prototype.connect = function () {
        var _this = this;
        return new Promise(function (ok, fail) {
            var connection = win.openDatabase(_this.options.database, '1.0', _this.options.database, 5 * 1024 * 1024);
            if (!connection)
                return fail();
            _this.databaseConnection = {
                id: 1,
                connection: connection,
                isTransactionActive: false
            };
            ok();
        });
    };
    /**
     * Closes connection with database.
     */
    WebSqlDriver.prototype.disconnect = function () {
        var _this = this;
        return new Promise(function (ok, fail) {
            var handler = function (err) { return err ? fail(err) : ok(); };
            if (!_this.databaseConnection)
                return fail(new ConnectionIsNotSetError("websql"));
            _this.databaseConnection.connection.close(handler);
        });
    };
    /**
     * Creates a query runner used for common queries.
     */
    WebSqlDriver.prototype.createQueryRunner = function () {
        return __awaiter(this, void 0, void 0, function () {
            var databaseConnection;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.databaseConnection)
                            return [2 /*return*/, Promise.reject(new ConnectionIsNotSetError("websql"))];
                        return [4 /*yield*/, this.retrieveDatabaseConnection()];
                    case 1:
                        databaseConnection = _a.sent();
                        return [2 /*return*/, new WebSqlQueryRunner(databaseConnection, this, this.logger)];
                }
            });
        });
    };
    /**
     * Access to the native implementation of the database.
     */
    WebSqlDriver.prototype.nativeInterface = function () {
        return {
            driver: this.websql,
            connection: this.databaseConnection ? this.databaseConnection.connection : undefined
        };
    };
    /**
     * Prepares given value to a value to be persisted, based on its column type and metadata.
     */
    WebSqlDriver.prototype.preparePersistentValue = function (value, column) {
        switch (column.type) {
            case ColumnTypes.BOOLEAN:
                return value === true ? 1 : 0;
            case ColumnTypes.DATE:
                return moment(value).format("YYYY-MM-DD");
            case ColumnTypes.TIME:
                return moment(value).format("HH:mm:ss");
            case ColumnTypes.DATETIME:
                return moment(value).format("YYYY-MM-DD HH:mm:ss");
            case ColumnTypes.JSON:
                return JSON.stringify(value);
            case ColumnTypes.SIMPLE_ARRAY:
                return value
                    .map(function (i) { return String(i); })
                    .join(",");
        }
        return value;
    };
    /**
     * Prepares given value to a value to be persisted, based on its column type or metadata.
     */
    WebSqlDriver.prototype.prepareHydratedValue = function (value, columnOrColumnType) {
        var type = columnOrColumnType instanceof ColumnMetadata ? columnOrColumnType.type : columnOrColumnType;
        switch (type) {
            case ColumnTypes.BOOLEAN:
                return value ? true : false;
            case ColumnTypes.DATE:
                if (value instanceof Date)
                    return value;
                return moment(value, "YYYY-MM-DD").toDate();
            case ColumnTypes.TIME:
                return moment(value, "HH:mm:ss").toDate();
            case ColumnTypes.DATETIME:
                if (value instanceof Date)
                    return value;
                return moment(value, "YYYY-MM-DD HH:mm:ss").toDate();
            case ColumnTypes.JSON:
                return JSON.parse(value);
            case ColumnTypes.SIMPLE_ARRAY:
                return value.split(",");
        }
        return value;
    };
    /**
     * Replaces parameters in the given sql with special escaping character
     * and an array of parameter names to be passed to a query.
     */
    WebSqlDriver.prototype.escapeQueryWithParameters = function (sql, parameters) {
        var _this = this;
        if (!parameters || !Object.keys(parameters).length)
            return [sql, []];
        var builtParameters = [];
        var keys = Object.keys(parameters).map(function (parameter) { return "(:" + parameter + "\\b)"; }).join("|");
        sql = sql.replace(new RegExp(keys, "g"), function (key) {
            var value = parameters[key.substr(1)];
            if (value instanceof Array) {
                return value.map(function (v) {
                    builtParameters.push(_this.prepareParameter(v));
                    return "?"; // + builtParameters.length;
                }).join(", ");
            }
            else {
                builtParameters.push(_this.prepareParameter(value));
            }
            return "?"; // + builtParameters.length;
        }); // todo: make replace only in value statements, otherwise problems
        return [sql, builtParameters];
    };
    /**
     * Prepares given parameter
     */
    WebSqlDriver.prototype.prepareParameter = function (param) {
        switch (typeof (param)) {
            case "boolean":
                return param === true ? 1 : 0;
            default:
                return param;
        }
    };
    /**
     * Escapes a column name.
     */
    WebSqlDriver.prototype.escapeColumnName = function (columnName) {
        return "\"" + columnName + "\"";
    };
    /**
     * Escapes an alias.
     */
    WebSqlDriver.prototype.escapeAliasName = function (aliasName) {
        return "\"" + aliasName + "\"";
    };
    /**
     * Escapes a table name.
     */
    WebSqlDriver.prototype.escapeTableName = function (tableName) {
        return "\"" + tableName + "\"";
    };
    // -------------------------------------------------------------------------
    // Protected Methods
    // -------------------------------------------------------------------------
    /**
     * Retrieves a new database connection.
     * If pooling is enabled then connection from the pool will be retrieved.
     * Otherwise active connection will be returned.
     */
    WebSqlDriver.prototype.retrieveDatabaseConnection = function () {
        if (this.databaseConnection)
            return Promise.resolve(this.databaseConnection);
        throw new ConnectionIsNotSetError("websql");
    };
    /**
     * If driver dependency is not given explicitly, then try to load it via "require".
     */
    WebSqlDriver.prototype.loadDependencies = function () {
        // if (!require)
        //     throw new DriverPackageLoadError();
        if (!win.openDatabase) {
            throw new DriverPackageNotInstalledError("WebSQL", "websql");
        }
    };
    return WebSqlDriver;
}());
export { WebSqlDriver };
//# sourceMappingURL=WebSqlDriver.js.map