import { QueryBuilder } from "../query-builder/QueryBuilder";
var LazyRelationsWrapper = (function () {
    function LazyRelationsWrapper(connection) {
        this.connection = connection;
    }
    LazyRelationsWrapper.prototype.wrap = function (object, relation) {
        var connection = this.connection;
        var index = "__" + relation.propertyName + "__";
        var loadIndex = "__load_" + relation.propertyName + "__";
        var resolveIndex = "__has_" + relation.propertyName + "__";
        Object.defineProperty(object, relation.propertyName, {
            get: function () {
                var _this = this;
                if (this[resolveIndex] === true)
                    return Promise.resolve(this[index]);
                if (this[loadIndex])
                    return this[loadIndex];
                var qb = new QueryBuilder(connection);
                if (relation.isManyToMany || relation.isOneToMany) {
                    if (relation.hasInverseSide) {
                        qb.select(relation.propertyName)
                            .from(relation.inverseRelation.entityMetadata.target, relation.propertyName)
                            .innerJoin(relation.propertyName + "." + relation.inverseRelation.propertyName, relation.entityMetadata.targetName);
                    }
                    else {
                        qb.select(relation.propertyName)
                            .from(relation.type, relation.propertyName)
                            .innerJoin(relation.junctionEntityMetadata.table.name, relation.junctionEntityMetadata.name, "ON", relation.junctionEntityMetadata.name + "." + relation.name + "=:" + relation.propertyName + "Id")
                            .setParameter(relation.propertyName + "Id", this[relation.referencedColumnName]);
                    }
                    this[loadIndex] = qb.getResults().then(function (results) {
                        _this[index] = results;
                        _this[resolveIndex] = true;
                        delete _this[loadIndex];
                        return _this[index];
                    }).catch(function (err) {
                        throw err;
                    });
                    return this[loadIndex];
                }
                else {
                    if (relation.hasInverseSide) {
                        qb.select(relation.propertyName)
                            .from(relation.inverseRelation.entityMetadata.target, relation.propertyName)
                            .innerJoin(relation.propertyName + "." + relation.inverseRelation.propertyName, relation.entityMetadata.targetName);
                    }
                    else {
                        // (ow) post.category<=>category.post
                        // loaded: category from post
                        qb.select(relation.propertyName) // category
                            .from(relation.type, relation.propertyName) // Category, category
                            .innerJoin(relation.entityMetadata.target, relation.entityMetadata.name, "ON", relation.entityMetadata.name + "." + relation.propertyName + "=:" + relation.propertyName + "Id") // Post, post, post.category = categoryId
                            .setParameter(relation.propertyName + "Id", this[relation.referencedColumnName]);
                    }
                    // console.log(qb.getSql());
                    this[loadIndex] = qb.getSingleResult().then(function (result) {
                        _this[index] = result;
                        _this[resolveIndex] = true;
                        delete _this[loadIndex];
                        return _this[index];
                    }).catch(function (err) {
                        throw err;
                    });
                    return this[loadIndex];
                }
            },
            set: function (promise) {
                var _this = this;
                if (promise instanceof Promise) {
                    promise.then(function (result) {
                        _this[index] = result;
                        _this[resolveIndex] = true;
                    });
                }
                else {
                    this[index] = promise;
                    this[resolveIndex] = true;
                }
            },
            configurable: true
        });
    };
    return LazyRelationsWrapper;
}());
export { LazyRelationsWrapper };
//# sourceMappingURL=LazyRelationsWrapper.js.map