/**
 * Performs logging of the events in TypeORM.
 */
var Logger = (function () {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    function Logger(options) {
        this.options = options;
    }
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    /**
     * Logs query and parameters used in it.
     */
    Logger.prototype.logQuery = function (query, parameters) {
        if (this.options.logQueries /*|| process.env.LOGGER_CLI_SCHEMA_SYNC*/)
            this.log("info", "executing query: " + query + (parameters && parameters.length ? " -- PARAMETERS: " + JSON.stringify(parameters) : ""));
    };
    /**
     * Logs query that failed.
     */
    Logger.prototype.logFailedQuery = function (query, parameters) {
        if (this.options.logQueries || this.options.logOnlyFailedQueries /*|| process.env.LOGGER_CLI_SCHEMA_SYNC*/)
            this.log("error", "query failed: " + query + (parameters && parameters.length ? " -- PARAMETERS: " + JSON.stringify(parameters) : ""));
    };
    /**
     * Logs failed query's error.
     */
    Logger.prototype.logQueryError = function (error) {
        if (this.options.logFailedQueryError /*|| process.env.LOGGER_CLI_SCHEMA_SYNC*/)
            this.log("error", "error during executing query:" + error);
    };
    /**
     * Logs events from the schema build process.
     */
    Logger.prototype.logSchemaBuild = function (message) {
        if (this.options.logSchemaCreation /*|| process.env.LOGGER_CLI_SCHEMA_SYNC*/)
            this.log("info", message);
    };
    /**
     * Perform logging using given logger, or by default to the console.
     * Log has its own level and message.
     */
    Logger.prototype.log = function (level, message) {
        if (!this.options)
            return;
        if (this.options.logger) {
            this.options.logger(level, message);
        }
        else {
            switch (level) {
                case "log":
                    console.log(message);
                    break;
                case "info":
                    console.info(message);
                    break;
                case "warn":
                    console.warn(message);
                    break;
                case "error":
                    console.error(message);
                    break;
            }
        }
    };
    return Logger;
}());
export { Logger };
//# sourceMappingURL=Logger.js.map