import { sha1 } from '../util/sha1';
import _ from "lodash";
/**
 * Naming strategy that is used by default.
 */
var DefaultNamingStrategy = (function () {
    function DefaultNamingStrategy() {
    }
    DefaultNamingStrategy.prototype.tableName = function (className, customName) {
        return customName ? customName : _.snakeCase(className);
    };
    DefaultNamingStrategy.prototype.columnName = function (propertyName, customName) {
        return customName ? customName : propertyName;
    };
    DefaultNamingStrategy.prototype.embeddedColumnName = function (embeddedPropertyName, columnPropertyName, columnCustomName) {
        return _.camelCase(embeddedPropertyName + "_" + (columnCustomName ? columnCustomName : columnPropertyName));
    };
    DefaultNamingStrategy.prototype.relationName = function (propertyName) {
        return propertyName;
    };
    DefaultNamingStrategy.prototype.indexName = function (customName, tableName, columns) {
        if (customName)
            return customName;
        var key = "ind_" + tableName + "_" + columns.join("_");
        return "ind_" + sha1.hash(key);
    };
    DefaultNamingStrategy.prototype.joinColumnInverseSideName = function (joinColumnName, propertyName) {
        if (joinColumnName)
            return joinColumnName;
        return propertyName;
    };
    DefaultNamingStrategy.prototype.joinTableName = function (firstTableName, secondTableName, firstPropertyName, secondPropertyName, firstColumnName, secondColumnName) {
        return _.snakeCase(firstTableName + "_" + firstPropertyName + "_" + secondTableName + "_" + secondColumnName);
    };
    DefaultNamingStrategy.prototype.joinTableColumnName = function (tableName, columnName, secondTableName, secondColumnName) {
        var column1 = _.camelCase(tableName + "_" + columnName);
        var column2 = _.camelCase(secondTableName + "_" + secondColumnName);
        return column1 === column2 ? column1 + "_1" : column1; // todo: do we still need _1 prefix?!
    };
    DefaultNamingStrategy.prototype.joinTableInverseColumnName = function (tableName, columnName, secondTableName, secondColumnName) {
        var column1 = _.camelCase(tableName + "_" + columnName);
        var column2 = _.camelCase(secondTableName + "_" + secondColumnName);
        return column1 === column2 ? column1 + "_2" : column1; // todo: do we still need _2 prefix?!
    };
    DefaultNamingStrategy.prototype.closureJunctionTableName = function (tableName) {
        return tableName + "_closure";
    };
    DefaultNamingStrategy.prototype.foreignKeyName = function (tableName, columnNames, referencedTableName, referencedColumnNames) {
        var key = tableName + "_" + columnNames.join("_") + "_" + referencedTableName + "_" + referencedColumnNames.join("_");
        return "fk_" + sha1.hash((key).substr(0, 27)); // todo: use crypto instead?
    };
    DefaultNamingStrategy.prototype.classTableInheritanceParentColumnName = function (parentTableName, parentTableIdPropertyName) {
        return _.camelCase(parentTableName + "_" + parentTableIdPropertyName);
    };
    /**
     * Adds prefix to the table.
     */
    DefaultNamingStrategy.prototype.prefixTableName = function (prefix, originalTableName) {
        return prefix + originalTableName;
    };
    return DefaultNamingStrategy;
}());
export { DefaultNamingStrategy };
//# sourceMappingURL=DefaultNamingStrategy.js.map