var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
/**
 * Executes PersistOperation in the given connection.
 */
var PersistOperationExecutor = (function () {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    function PersistOperationExecutor(driver, entityMetadatas, broadcaster, queryRunner) {
        this.driver = driver;
        this.entityMetadatas = entityMetadatas;
        this.broadcaster = broadcaster;
        this.queryRunner = queryRunner;
    }
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    /**
     * Executes given persist operation.
     */
    PersistOperationExecutor.prototype.executePersistOperation = function (persistOperation) {
        var _this = this;
        var isTransactionStartedByItself = false;
        // persistOperation.log();
        return Promise.resolve()
            .then(function () { return _this.broadcastBeforeEvents(persistOperation); })
            .then(function () {
            if (!_this.queryRunner.isTransactionActive()) {
                isTransactionStartedByItself = true;
                return _this.queryRunner.beginTransaction();
            }
            return undefined;
        })
            .then(function () { return _this.executeInsertOperations(persistOperation); })
            .then(function () { return _this.executeInsertClosureTableOperations(persistOperation); })
            .then(function () { return _this.executeUpdateTreeLevelOperations(persistOperation); })
            .then(function () { return _this.executeInsertJunctionsOperations(persistOperation); })
            .then(function () { return _this.executeRemoveJunctionsOperations(persistOperation); })
            .then(function () { return _this.executeRemoveRelationOperations(persistOperation); })
            .then(function () { return _this.executeUpdateRelationsOperations(persistOperation); })
            .then(function () { return _this.executeUpdateInverseRelationsOperations(persistOperation); })
            .then(function () { return _this.executeUpdateOperations(persistOperation); })
            .then(function () { return _this.executeRemoveOperations(persistOperation); })
            .then(function () {
            if (isTransactionStartedByItself === true)
                return _this.queryRunner.commitTransaction();
            return Promise.resolve();
        })
            .then(function () { return _this.updateIdsOfInsertedEntities(persistOperation); })
            .then(function () { return _this.updateIdsOfRemovedEntities(persistOperation); })
            .then(function () { return _this.updateSpecialColumnsInEntities(persistOperation); })
            .then(function () { return _this.broadcastAfterEvents(persistOperation); })
            .catch(function (error) {
            if (isTransactionStartedByItself === true) {
                return _this.queryRunner.rollbackTransaction()
                    .then(function () {
                    throw error;
                })
                    .catch(function () {
                    throw error;
                });
            }
            throw error;
        });
    };
    // -------------------------------------------------------------------------
    // Private Methods
    // -------------------------------------------------------------------------
    /**
     * Broadcast all before persistment events - beforeInsert, beforeUpdate and beforeRemove events.
     */
    PersistOperationExecutor.prototype.broadcastBeforeEvents = function (persistOperation) {
        var _this = this;
        var insertEvents = persistOperation.inserts.map(function (insertOperation) {
            var persistedEntityWithId = persistOperation.allPersistedEntities.find(function (e) { return e.entity === insertOperation.entity; });
            if (!persistedEntityWithId)
                throw new Error("Persisted entity was not found");
            return _this.broadcaster.broadcastBeforeInsertEvent(persistedEntityWithId.entity);
        });
        var updateEvents = persistOperation.updates.map(function (updateOperation) {
            var persistedEntityWithId = persistOperation.allPersistedEntities.find(function (e) { return e.entity === updateOperation.entity; });
            if (!persistedEntityWithId)
                throw new Error("Persisted entity was not found");
            return _this.broadcaster.broadcastBeforeUpdateEvent(persistedEntityWithId.entity, updateOperation.columns);
        });
        var removeEvents = persistOperation.removes.map(function (removeOperation) {
            // we can send here only dbEntity, not entity from the persisted object, since entity from the persisted
            // object does not exist anymore - its removed, and there is no way to find this removed object
            return _this.broadcaster.broadcastBeforeRemoveEvent(removeOperation.entity, removeOperation.entityId);
        });
        return Promise.all(insertEvents)
            .then(function () { return Promise.all(updateEvents); })
            .then(function () { return Promise.all(removeEvents); }); // todo: do we really should send it in order?
    };
    /**
     * Broadcast all after persistment events - afterInsert, afterUpdate and afterRemove events.
     */
    PersistOperationExecutor.prototype.broadcastAfterEvents = function (persistOperation) {
        var _this = this;
        var insertEvents = persistOperation.inserts.map(function (insertOperation) {
            var persistedEntity = persistOperation.allPersistedEntities.find(function (e) { return e.entity === insertOperation.entity; });
            if (!persistedEntity)
                throw new Error("Persisted entity was not found");
            return _this.broadcaster.broadcastAfterInsertEvent(persistedEntity.entity);
        });
        var updateEvents = persistOperation.updates.map(function (updateOperation) {
            var persistedEntityWithId = persistOperation.allPersistedEntities.find(function (e) { return e.entity === updateOperation.entity; });
            if (!persistedEntityWithId)
                throw new Error("Persisted entity was not found");
            return _this.broadcaster.broadcastAfterUpdateEvent(persistedEntityWithId.entity, updateOperation.columns);
        });
        var removeEvents = persistOperation.removes.map(function (removeOperation) {
            // we can send here only dbEntity, not entity from the persisted object, since entity from the persisted
            // object does not exist anymore - its removed, and there is no way to find this removed object
            return _this.broadcaster.broadcastAfterRemoveEvent(removeOperation.entity, removeOperation.entityId);
        });
        return Promise.all(insertEvents)
            .then(function () { return Promise.all(updateEvents); })
            .then(function () { return Promise.all(removeEvents); }); // todo: do we really should send it in order?
    };
    /**
     * Executes insert operations.
     */
    PersistOperationExecutor.prototype.executeInsertOperations = function (persistOperation) {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, Promise.all(persistOperation.inserts.map(function (operation) { return __awaiter(_this, void 0, void 0, function () {
                            return __generator(this, function (_a) {
                                return [2 /*return*/, this.insert(operation)];
                            });
                        }); }))];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Executes insert operations for closure tables.
     */
    PersistOperationExecutor.prototype.executeInsertClosureTableOperations = function (persistOperation) {
        var _this = this;
        var promises = persistOperation.inserts
            .filter(function (operation) {
            var metadata = _this.entityMetadatas.findByTarget(operation.target);
            return metadata.table.isClosure;
        })
            .map(function (operation) {
            var relationsUpdateMap = _this.findUpdateOperationForEntity(persistOperation.updatesByRelations, persistOperation.inserts, operation.entity);
            return _this.insertIntoClosureTable(operation, relationsUpdateMap).then(function (level) {
                operation.treeLevel = level;
            });
        });
        return Promise.all(promises);
    };
    /**
     * Executes update tree level operations in inserted entities right after data into closure table inserted.
     */
    PersistOperationExecutor.prototype.executeUpdateTreeLevelOperations = function (persistOperation) {
        var _this = this;
        return Promise.all(persistOperation.inserts.map(function (operation) {
            return _this.updateTreeLevel(operation);
        }));
    };
    /**
     * Executes insert junction operations.
     */
    PersistOperationExecutor.prototype.executeInsertJunctionsOperations = function (persistOperation) {
        var _this = this;
        return Promise.all(persistOperation.junctionInserts.map(function (junctionOperation) {
            return _this.insertJunctions(junctionOperation, persistOperation.inserts);
        }));
    };
    /**
     * Executes remove junction operations.
     */
    PersistOperationExecutor.prototype.executeRemoveJunctionsOperations = function (persistOperation) {
        var _this = this;
        return Promise.all(persistOperation.junctionRemoves.map(function (junctionOperation) {
            return _this.removeJunctions(junctionOperation);
        }));
    };
    /**
     * Executes update relations operations.
     */
    PersistOperationExecutor.prototype.executeUpdateRelationsOperations = function (persistOperation) {
        var _this = this;
        return Promise.all(persistOperation.updatesByRelations.map(function (updateByRelation) {
            return _this.updateByRelation(updateByRelation, persistOperation.inserts);
        }));
    };
    /**
     * Executes update relations operations.
     */
    PersistOperationExecutor.prototype.executeUpdateInverseRelationsOperations = function (persistOperation) {
        var _this = this;
        return Promise.all(persistOperation.updatesByInverseRelations.map(function (updateInverseOperation) {
            return _this.updateInverseRelation(updateInverseOperation, persistOperation.inserts);
        }));
    };
    /**
     * Executes update operations.
     */
    PersistOperationExecutor.prototype.executeUpdateOperations = function (persistOperation) {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, Promise.all(persistOperation.updates.map(function (updateOperation) {
                            return _this.update(updateOperation);
                        }))];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Executes remove relations operations.
     */
    PersistOperationExecutor.prototype.executeRemoveRelationOperations = function (persistOperation) {
        var _this = this;
        return Promise.all(persistOperation.removes
            .filter(function (operation) {
            return !!(operation.relation && !operation.relation.isManyToMany && !operation.relation.isOneToMany);
        })
            .map(function (operation) {
            return _this.updateDeletedRelations(operation);
        }));
    };
    /**
     * Executes remove operations.
     */
    PersistOperationExecutor.prototype.executeRemoveOperations = function (persistOperation) {
        var _this = this;
        return Promise.all(persistOperation.removes.map(function (operation) {
            return _this.delete(operation.target, operation.entity);
        }));
    };
    /**
     * Updates all ids of the inserted entities.
     */
    PersistOperationExecutor.prototype.updateIdsOfInsertedEntities = function (persistOperation) {
        var _this = this;
        persistOperation.inserts.forEach(function (insertOperation) {
            var metadata = _this.entityMetadatas.findByTarget(insertOperation.target);
            metadata.primaryColumns.forEach(function (primaryColumn) {
                if (insertOperation.entityId)
                    insertOperation.entity[primaryColumn.propertyName] = insertOperation.entityId[primaryColumn.propertyName];
            });
            metadata.parentPrimaryColumns.forEach(function (primaryColumn) {
                if (insertOperation.entityId)
                    insertOperation.entity[primaryColumn.propertyName] = insertOperation.entityId[primaryColumn.propertyName];
            });
        });
    };
    /**
     * Updates all special columns of the saving entities (create date, update date, versioning).
     */
    PersistOperationExecutor.prototype.updateSpecialColumnsInEntities = function (persistOperation) {
        var _this = this;
        persistOperation.inserts.forEach(function (insertOperation) {
            var metadata = _this.entityMetadatas.findByTarget(insertOperation.target);
            if (metadata.hasUpdateDateColumn)
                insertOperation.entity[metadata.updateDateColumn.propertyName] = insertOperation.date;
            if (metadata.hasCreateDateColumn)
                insertOperation.entity[metadata.createDateColumn.propertyName] = insertOperation.date;
            if (metadata.hasVersionColumn)
                insertOperation.entity[metadata.versionColumn.propertyName]++;
            if (metadata.hasTreeLevelColumn) {
                // const parentEntity = insertOperation.entity[metadata.treeParentMetadata.propertyName];
                // const parentLevel = parentEntity ? (parentEntity[metadata.treeLevelColumn.propertyName] || 0) : 0;
                insertOperation.entity[metadata.treeLevelColumn.propertyName] = insertOperation.treeLevel;
            }
            /*if (metadata.hasTreeChildrenCountColumn) {
                insertOperation.entity[metadata.treeChildrenCountColumn.propertyName] = 0;
            }*/
        });
        persistOperation.updates.forEach(function (updateOperation) {
            var metadata = _this.entityMetadatas.findByTarget(updateOperation.target);
            if (metadata.hasUpdateDateColumn)
                updateOperation.entity[metadata.updateDateColumn.propertyName] = updateOperation.date;
            if (metadata.hasCreateDateColumn)
                updateOperation.entity[metadata.createDateColumn.propertyName] = updateOperation.date;
            if (metadata.hasVersionColumn)
                updateOperation.entity[metadata.versionColumn.propertyName]++;
        });
    };
    /**
     * Removes all ids of the removed entities.
     */
    PersistOperationExecutor.prototype.updateIdsOfRemovedEntities = function (persistOperation) {
        var _this = this;
        persistOperation.removes.forEach(function (removeOperation) {
            var metadata = _this.entityMetadatas.findByTarget(removeOperation.target);
            var removedEntity = persistOperation.allPersistedEntities.find(function (allNewEntity) {
                return allNewEntity.entityTarget === removeOperation.target && allNewEntity.compareId(metadata.getEntityIdMap(removeOperation.entity));
            });
            if (removedEntity) {
                metadata.primaryColumns.forEach(function (primaryColumn) {
                    removedEntity.entity[primaryColumn.propertyName] = undefined;
                });
            }
        });
    };
    PersistOperationExecutor.prototype.findUpdateOperationForEntity = function (operations, insertOperations, target) {
        // we are using firstPrimaryColumn here because this method is used only in executeInsertClosureTableOperations method
        // which means only for tree tables, but multiple primary keys are not supported in tree tables
        var _this = this;
        var updateMap = {};
        operations
            .forEach(function (operation) {
            var metadata = _this.entityMetadatas.findByTarget(operation.insertOperation.target);
            var relatedInsertOperation = insertOperations.find(function (o) { return o.entity === operation.targetEntity; });
            // todo: looks like first primary column should not be used there for two reasons:
            // 1. there can be multiple primary columns, which one is mapped in the relation
            // 2. parent primary column
            // join column should be used instead
            if (operation.updatedRelation.isOneToMany) {
                var idInInserts = relatedInsertOperation && relatedInsertOperation.entityId ? relatedInsertOperation.entityId[metadata.firstPrimaryColumn.propertyName] : null;
                if (operation.insertOperation.entity === target)
                    updateMap[operation.updatedRelation.inverseRelation.propertyName] = operation.targetEntity[metadata.firstPrimaryColumn.propertyName] || idInInserts;
            }
            else {
                if (operation.targetEntity === target && operation.insertOperation.entityId)
                    updateMap[operation.updatedRelation.propertyName] = operation.insertOperation.entityId[metadata.firstPrimaryColumn.propertyName];
            }
        });
        return updateMap;
    };
    PersistOperationExecutor.prototype.updateByRelation = function (operation, insertOperations) {
        if (!operation.insertOperation.entityId)
            throw new Error("insert operation does not have entity id");
        var tableName, relationName, relationId, idColumn, id, updateMap;
        var relatedInsertOperation = insertOperations.find(function (o) { return o.entity === operation.targetEntity; });
        if (operation.updatedRelation.isOneToMany || operation.updatedRelation.isOneToOneNotOwner) {
            var metadata = this.entityMetadatas.findByTarget(operation.insertOperation.target);
            var idInInserts = relatedInsertOperation && relatedInsertOperation.entityId ? relatedInsertOperation.entityId[metadata.firstPrimaryColumn.propertyName] : null; // todo: use join column instead of primary column here
            tableName = metadata.table.name;
            relationName = operation.updatedRelation.inverseRelation.name;
            relationId = operation.targetEntity[metadata.firstPrimaryColumn.propertyName] || idInInserts; // todo: make sure idInInserts is always a map
            updateMap = metadata.transformIdMapToColumnNames(operation.insertOperation.entityId);
        }
        else {
            var metadata = this.entityMetadatas.findByTarget(operation.entityTarget);
            var idInInserts = undefined;
            if (relatedInsertOperation && relatedInsertOperation.entityId) {
                idInInserts = (_a = {}, _a[metadata.firstPrimaryColumn.propertyName] = relatedInsertOperation.entityId[metadata.firstPrimaryColumn.propertyName], _a);
            } // todo: use join column instead of primary column here
            tableName = metadata.table.name;
            relationName = operation.updatedRelation.name;
            relationId = operation.insertOperation.entityId[metadata.firstPrimaryColumn.propertyName]; // todo: make sure entityId is always a map
            // idColumn = metadata.primaryColumn.name;
            // id = operation.targetEntity[metadata.primaryColumn.propertyName] || idInInserts;
            updateMap = metadata.getEntityIdColumnMap(operation.targetEntity) || idInInserts; // todo: make sure idInInserts always object even when id is single!!!
        }
        if (!updateMap)
            throw new Error("Cannot execute update by relation operation, because cannot find update criteria");
        return this.queryRunner.update(tableName, (_b = {}, _b[relationName] = relationId, _b), updateMap);
        var _a, _b;
    };
    PersistOperationExecutor.prototype.updateInverseRelation = function (operation, insertOperations) {
        var targetEntityMetadata = this.entityMetadatas.findByTarget(operation.entityTarget);
        var fromEntityMetadata = this.entityMetadatas.findByTarget(operation.fromEntityTarget);
        var tableName = targetEntityMetadata.table.name;
        var targetRelation = operation.fromRelation.inverseRelation;
        var updateMap = targetEntityMetadata.getEntityIdColumnMap(operation.targetEntity);
        if (!updateMap)
            return; // todo: is return correct here?
        var fromEntityInsertOperation = insertOperations.find(function (o) { return o.entity === operation.fromEntity; });
        var targetEntityId; // todo: better do it during insertion - pass UpdateByInverseSideOperation[] to insert and do it there
        if (operation.operationType === "remove") {
            targetEntityId = null;
        }
        else {
            if (fromEntityInsertOperation && fromEntityInsertOperation.entityId && targetRelation.joinColumn.referencedColumn === fromEntityMetadata.firstPrimaryColumn) {
                targetEntityId = fromEntityInsertOperation.entityId[fromEntityMetadata.firstPrimaryColumn.name];
            }
            else {
                targetEntityId = operation.fromEntity[targetRelation.joinColumn.referencedColumn.name];
            }
        }
        return this.queryRunner.update(tableName, (_a = {}, _a[targetRelation.name] = targetEntityId, _a), updateMap);
        var _a;
    };
    PersistOperationExecutor.prototype.update = function (updateOperation) {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            var entity, metadata, valueMaps, valueMap, valueMap, valueMap, valueMap;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        entity = updateOperation.entity;
                        metadata = this.entityMetadatas.findByTarget(updateOperation.target);
                        valueMaps = [];
                        updateOperation.columns.forEach(function (column) {
                            if (!column.entityTarget)
                                return;
                            var metadata = _this.entityMetadatas.findByTarget(column.entityTarget);
                            var valueMap = valueMaps.find(function (valueMap) { return valueMap.tableName === metadata.table.name; });
                            if (!valueMap) {
                                valueMap = { tableName: metadata.table.name, metadata: metadata, values: {} };
                                valueMaps.push(valueMap);
                            }
                            valueMap.values[column.name] = _this.driver.preparePersistentValue(column.getEntityValue(entity), column);
                        });
                        updateOperation.relations.forEach(function (relation) {
                            var metadata = _this.entityMetadatas.findByTarget(relation.entityTarget);
                            var valueMap = valueMaps.find(function (valueMap) { return valueMap.tableName === metadata.table.name; });
                            if (!valueMap) {
                                valueMap = { tableName: metadata.table.name, metadata: metadata, values: {} };
                                valueMaps.push(valueMap);
                            }
                            var value = _this.getEntityRelationValue(relation, entity);
                            valueMap.values[relation.name] = value !== null && value !== undefined ? value[relation.inverseEntityMetadata.firstPrimaryColumn.propertyName] : null; // todo: should not have a call to primaryColumn, instead join column metadata should be used
                        });
                        // if number of updated columns = 0 no need to update updated date and version columns
                        if (Object.keys(valueMaps).length === 0)
                            return [2 /*return*/];
                        if (metadata.hasUpdateDateColumn) {
                            valueMap = valueMaps.find(function (valueMap) { return valueMap.tableName === metadata.table.name; });
                            if (!valueMap) {
                                valueMap = { tableName: metadata.table.name, metadata: metadata, values: {} };
                                valueMaps.push(valueMap);
                            }
                            valueMap.values[metadata.updateDateColumn.name] = this.driver.preparePersistentValue(new Date(), metadata.updateDateColumn);
                        }
                        if (metadata.hasVersionColumn) {
                            valueMap = valueMaps.find(function (valueMap) { return valueMap.tableName === metadata.table.name; });
                            if (!valueMap) {
                                valueMap = { tableName: metadata.table.name, metadata: metadata, values: {} };
                                valueMaps.push(valueMap);
                            }
                            valueMap.values[metadata.versionColumn.name] = this.driver.preparePersistentValue(entity[metadata.versionColumn.propertyName] + 1, metadata.versionColumn);
                        }
                        if (metadata.parentEntityMetadata) {
                            if (metadata.parentEntityMetadata.hasUpdateDateColumn) {
                                valueMap = valueMaps.find(function (valueMap) { return valueMap.tableName === metadata.parentEntityMetadata.table.name; });
                                if (!valueMap) {
                                    valueMap = { tableName: metadata.parentEntityMetadata.table.name, metadata: metadata.parentEntityMetadata, values: {} };
                                    valueMaps.push(valueMap);
                                }
                                valueMap.values[metadata.parentEntityMetadata.updateDateColumn.name] = this.driver.preparePersistentValue(new Date(), metadata.parentEntityMetadata.updateDateColumn);
                            }
                            if (metadata.parentEntityMetadata.hasVersionColumn) {
                                valueMap = valueMaps.find(function (valueMap) { return valueMap.tableName === metadata.parentEntityMetadata.table.name; });
                                if (!valueMap) {
                                    valueMap = { tableName: metadata.parentEntityMetadata.table.name, metadata: metadata.parentEntityMetadata, values: {} };
                                    valueMaps.push(valueMap);
                                }
                                valueMap.values[metadata.parentEntityMetadata.versionColumn.name] = this.driver.preparePersistentValue(entity[metadata.parentEntityMetadata.versionColumn.propertyName] + 1, metadata.parentEntityMetadata.versionColumn);
                            }
                        }
                        return [4 /*yield*/, Promise.all(valueMaps.map(function (valueMap) {
                                var conditions = {};
                                if (valueMap.metadata.parentEntityMetadata && valueMap.metadata.parentEntityMetadata.inheritanceType === "class-table") {
                                    valueMap.metadata.parentIdColumns.forEach(function (column) {
                                        conditions[column.name] = entity[column.propertyName];
                                    });
                                }
                                else {
                                    valueMap.metadata.primaryColumns.forEach(function (column) {
                                        conditions[column.name] = entity[column.propertyName];
                                    });
                                }
                                return _this.queryRunner.update(valueMap.tableName, valueMap.values, conditions);
                            }))];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    PersistOperationExecutor.prototype.updateDeletedRelations = function (removeOperation) {
        if (!removeOperation.fromEntityId)
            throw new Error("remove operation does not have entity id");
        if (removeOperation.relation) {
            return this.queryRunner.update(removeOperation.fromMetadata.table.name, (_a = {}, _a[removeOperation.relation.name] = null, _a), removeOperation.fromEntityId);
        }
        throw new Error("Remove operation relation is not set"); // todo: find out how its possible
        var _a;
    };
    PersistOperationExecutor.prototype.delete = function (target, entity) {
        return __awaiter(this, void 0, void 0, function () {
            var metadata, parentConditions_1, childConditions_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        metadata = this.entityMetadatas.findByTarget(target);
                        if (!metadata.parentEntityMetadata) return [3 /*break*/, 3];
                        parentConditions_1 = {};
                        metadata.parentPrimaryColumns.forEach(function (column) {
                            parentConditions_1[column.name] = entity[column.propertyName];
                        });
                        return [4 /*yield*/, this.queryRunner.delete(metadata.parentEntityMetadata.table.name, parentConditions_1)];
                    case 1:
                        _a.sent();
                        childConditions_1 = {};
                        metadata.primaryColumnsWithParentIdColumns.forEach(function (column) {
                            childConditions_1[column.name] = entity[column.propertyName];
                        });
                        return [4 /*yield*/, this.queryRunner.delete(metadata.table.name, childConditions_1)];
                    case 2:
                        _a.sent();
                        return [3 /*break*/, 5];
                    case 3: return [4 /*yield*/, this.queryRunner.delete(metadata.table.name, metadata.getEntityIdColumnMap(entity))];
                    case 4:
                        _a.sent();
                        _a.label = 5;
                    case 5: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Inserts an entity from the given insert operation into the database.
     * If entity has an generated column, then after saving new generated value will be stored to the InsertOperation.
     * If entity uses class-table-inheritance, then multiple inserts may by performed to save all entities.
     */
    PersistOperationExecutor.prototype.insert = function (operation) {
        return __awaiter(this, void 0, void 0, function () {
            var metadata, generatedId, parentValuesMap, childValuesMap, secondGeneratedId, valuesMap, _a, _b;
            return __generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        metadata = this.entityMetadatas.findByTarget(operation.target);
                        if (!metadata.table.isClassTableChild) return [3 /*break*/, 3];
                        parentValuesMap = this.collectColumnsAndValues(metadata.parentEntityMetadata, operation.entity, operation.date, undefined, metadata.discriminatorValue);
                        return [4 /*yield*/, this.queryRunner.insert(metadata.parentEntityMetadata.table.name, parentValuesMap, metadata.parentEntityMetadata.generatedColumnIfExist)];
                    case 1:
                        generatedId = _c.sent();
                        childValuesMap = this.collectColumnsAndValues(metadata, operation.entity, operation.date, generatedId);
                        return [4 /*yield*/, this.queryRunner.insert(metadata.table.name, childValuesMap, metadata.generatedColumnIfExist)];
                    case 2:
                        secondGeneratedId = _c.sent();
                        if (!generatedId && secondGeneratedId)
                            generatedId = secondGeneratedId;
                        return [3 /*break*/, 5];
                    case 3:
                        valuesMap = this.collectColumnsAndValues(metadata, operation.entity, operation.date);
                        return [4 /*yield*/, this.queryRunner.insert(metadata.table.name, valuesMap, metadata.generatedColumnIfExist)];
                    case 4:
                        generatedId = _c.sent();
                        _c.label = 5;
                    case 5:
                        // if there is a generated column and we have a generated id then store it in the insert operation for further use
                        if (metadata.parentEntityMetadata && metadata.parentEntityMetadata.hasGeneratedColumn && generatedId) {
                            operation.entityId = (_a = {}, _a[metadata.parentEntityMetadata.generatedColumn.propertyName] = generatedId, _a);
                        }
                        else if (metadata.hasGeneratedColumn && generatedId) {
                            operation.entityId = (_b = {}, _b[metadata.generatedColumn.propertyName] = generatedId, _b);
                        }
                        return [2 /*return*/];
                }
            });
        });
    };
    PersistOperationExecutor.prototype.collectColumnsAndValues = function (metadata, entity, date, parentIdColumnValue, discriminatorValue) {
        var _this = this;
        var columns = metadata.columns
            .filter(function (column) { return !column.isVirtual && !column.isParentId && !column.isDiscriminator && column.hasEntityValue(entity); });
        var columnNames = columns.map(function (column) { return column.name; });
        var values = columns.map(function (column) { return _this.driver.preparePersistentValue(column.getEntityValue(entity), column); });
        var relationColumns = metadata.relations
            .filter(function (relation) { return !relation.isManyToMany && relation.isOwning && !!relation.inverseEntityMetadata; })
            .filter(function (relation) { return entity.hasOwnProperty(relation.propertyName); })
            .map(function (relation) { return relation.name; });
        var relationValues = metadata.relations
            .filter(function (relation) { return !relation.isManyToMany && relation.isOwning && !!relation.inverseEntityMetadata; })
            .filter(function (relation) { return entity.hasOwnProperty(relation.propertyName); })
            .map(function (relation) {
            var value = _this.getEntityRelationValue(relation, entity);
            if (value !== null && value !== undefined)
                return value[relation.inverseEntityMetadata.firstPrimaryColumn.propertyName]; // todo: it should be get by field set in join column in the relation metadata
            return value;
        });
        var allColumns = columnNames.concat(relationColumns);
        var allValues = values.concat(relationValues);
        if (metadata.hasCreateDateColumn) {
            allColumns.push(metadata.createDateColumn.name);
            allValues.push(this.driver.preparePersistentValue(date, metadata.createDateColumn));
        }
        if (metadata.hasUpdateDateColumn) {
            allColumns.push(metadata.updateDateColumn.name);
            allValues.push(this.driver.preparePersistentValue(date, metadata.updateDateColumn));
        }
        if (metadata.hasVersionColumn) {
            allColumns.push(metadata.versionColumn.name);
            allValues.push(this.driver.preparePersistentValue(1, metadata.versionColumn));
        }
        if (metadata.hasDiscriminatorColumn) {
            allColumns.push(metadata.discriminatorColumn.name);
            allValues.push(this.driver.preparePersistentValue(discriminatorValue || metadata.discriminatorValue, metadata.discriminatorColumn));
        }
        if (metadata.hasTreeLevelColumn && metadata.hasTreeParentRelation) {
            var parentEntity = entity[metadata.treeParentRelation.propertyName];
            var parentLevel = parentEntity ? (parentEntity[metadata.treeLevelColumn.propertyName] || 0) : 0;
            allColumns.push(metadata.treeLevelColumn.name);
            allValues.push(parentLevel + 1);
        }
        if (metadata.parentEntityMetadata && metadata.hasParentIdColumn) {
            allColumns.push(metadata.parentIdColumn.name); // todo: should be array of primary keys
            allValues.push(parentIdColumnValue || entity[metadata.parentEntityMetadata.firstPrimaryColumn.propertyName]); // todo: should be array of primary keys
        }
        return this.zipObject(allColumns, allValues);
    };
    PersistOperationExecutor.prototype.insertIntoClosureTable = function (operation, updateMap) {
        // here we can only support to work only with single primary key entities
        var entity = operation.entity;
        var metadata = this.entityMetadatas.findByTarget(operation.target);
        var parentEntity = entity[metadata.treeParentRelation.propertyName];
        var parentEntityId = 0;
        if (parentEntity && parentEntity[metadata.firstPrimaryColumn.propertyName]) {
            parentEntityId = parentEntity[metadata.firstPrimaryColumn.propertyName];
        }
        else if (updateMap && updateMap[metadata.treeParentRelation.propertyName]) {
            parentEntityId = updateMap[metadata.treeParentRelation.propertyName];
        }
        if (!operation.entityId)
            throw new Error("operation does not have entity id");
        // todo: this code does not take in count a primary column from the parent entity metadata
        return this.queryRunner.insertIntoClosureTable(metadata.closureJunctionTable.table.name, operation.entityId[metadata.firstPrimaryColumn.propertyName], parentEntityId, metadata.hasTreeLevelColumn);
    };
    PersistOperationExecutor.prototype.updateTreeLevel = function (operation) {
        var metadata = this.entityMetadatas.findByTarget(operation.target);
        if (metadata.hasTreeLevelColumn && operation.treeLevel) {
            if (!operation.entityId)
                throw new Error("remove operation does not have entity id");
            var values = (_a = {}, _a[metadata.treeLevelColumn.name] = operation.treeLevel, _a);
            return this.queryRunner.update(metadata.table.name, values, operation.entityId);
        }
        return Promise.resolve();
        var _a;
    };
    PersistOperationExecutor.prototype.insertJunctions = function (junctionOperation, insertOperations) {
        // I think here we can only support to work only with single primary key entities
        var junctionMetadata = junctionOperation.metadata;
        var metadata1 = this.entityMetadatas.findByTarget(junctionOperation.entity1Target);
        var metadata2 = this.entityMetadatas.findByTarget(junctionOperation.entity2Target);
        var columns = junctionMetadata.columns.map(function (column) { return column.name; });
        var insertOperation1 = insertOperations.find(function (o) { return o.entity === junctionOperation.entity1; });
        var insertOperation2 = insertOperations.find(function (o) { return o.entity === junctionOperation.entity2; });
        // todo: firstPrimaryColumn should not be used there! use join column's properties instead!
        var id1 = junctionOperation.entity1[metadata1.firstPrimaryColumn.propertyName];
        var id2 = junctionOperation.entity2[metadata2.firstPrimaryColumn.propertyName];
        if (!id1) {
            if (insertOperation1 && insertOperation1.entityId) {
                id1 = insertOperation1.entityId[metadata1.firstPrimaryColumn.propertyName];
            }
            else {
                throw new Error("Cannot insert object of " + junctionOperation.entity1.constructor.name + " type. Looks like its not persisted yet, or cascades are not set on the relation.");
            }
        }
        if (!id2) {
            if (insertOperation2 && insertOperation2.entityId) {
                id2 = insertOperation2.entityId[metadata2.firstPrimaryColumn.propertyName];
            }
            else {
                throw new Error("Cannot insert object of " + junctionOperation.entity2.constructor.name + " type. Looks like its not persisted yet, or cascades are not set on the relation.");
            }
        }
        var values;
        // order may differ, find solution (column.table to compare with entity metadata table?)
        if (metadata1.table === junctionMetadata.foreignKeys[0].referencedTable) {
            values = [id1, id2];
        }
        else {
            values = [id2, id1];
        }
        return this.queryRunner.insert(junctionMetadata.table.name, this.zipObject(columns, values));
    };
    PersistOperationExecutor.prototype.removeJunctions = function (junctionOperation) {
        // I think here we can only support to work only with single primary key entities
        var junctionMetadata = junctionOperation.metadata;
        var metadata1 = this.entityMetadatas.findByTarget(junctionOperation.entity1Target);
        var metadata2 = this.entityMetadatas.findByTarget(junctionOperation.entity2Target);
        var columns = junctionMetadata.columns.map(function (column) { return column.name; });
        var id1 = junctionOperation.entity1[metadata1.firstPrimaryColumn.propertyName];
        var id2 = junctionOperation.entity2[metadata2.firstPrimaryColumn.propertyName];
        return this.queryRunner.delete(junctionMetadata.table.name, (_a = {}, _a[columns[0]] = id1, _a[columns[1]] = id2, _a));
        var _a;
    };
    PersistOperationExecutor.prototype.zipObject = function (keys, values) {
        return keys.reduce(function (object, column, index) {
            object[column] = values[index];
            return object;
        }, {});
    };
    PersistOperationExecutor.prototype.getEntityRelationValue = function (relation, entity) {
        return relation.isLazy ? entity["__" + relation.propertyName + "__"] : entity[relation.propertyName];
    };
    return PersistOperationExecutor;
}());
export { PersistOperationExecutor };
//# sourceMappingURL=PersistOperationExecutor.js.map